@@ -100,7 +100,26 @@ struct _RsttoMainWindowPriv
                 GtkWidget *menu_item_thumbnail_viewer_vertical;
                 GtkWidget *menu_item_thumbnail_viewer_hide;
             } show_thumbnail_viewer;
+            GtkWidget *menu_item_separator_1;
 
+            GtkWidget *menu_item_zooming;
+            struct {
+                GtkWidget *menu;
+                GtkWidget *menu_item_zoom_in;
+                GtkWidget *menu_item_zoom_out;
+                GtkWidget *menu_item_zoom_100;
+                GtkWidget *menu_item_zoom_fit;
+                GtkWidget *menu_item_zoom_box;
+            } zooming;
+
+            GtkWidget *menu_item_rotate;
+            struct {
+                GtkWidget *menu;
+                GtkWidget *menu_item_rotate_cw;
+                GtkWidget *menu_item_rotate_ccw;
+            } rotate;
+
+            GtkWidget *menu_item_separator_2;
             GtkWidget *menu_item_fullscreen;
         } view;
 
@@ -195,6 +214,11 @@ cb_rstto_main_window_zoom_100(GtkWidget *widget, RsttoMainWindow *window);
 static void
 cb_rstto_main_window_zoom_fit(GtkWidget *widget, RsttoMainWindow *window);
 
+static void
+cb_rstto_main_window_rotate_cw(GtkWidget *widget, RsttoMainWindow *window);
+static void
+cb_rstto_main_window_rotate_ccw(GtkWidget *widget, RsttoMainWindow *window);
+
 static void
 cb_rstto_main_window_open_file(GtkWidget *widget, RsttoMainWindow *window);
 static void
@@ -352,8 +376,16 @@ rstto_main_window_init(RsttoMainWindow *window)
     gtk_menu_set_accel_group(GTK_MENU(window->priv->menus.view.menu), accel_group);
     gtk_menu_item_set_submenu(GTK_MENU_ITEM(window->priv->menus.menu_item_view), window->priv->menus.view.menu);
 
-    window->priv->menus.view.menu_item_show_toolbar = gtk_check_menu_item_new_with_mnemonic(_("Show Toolbar"));
-    window->priv->menus.view.menu_item_show_thumbnail_viewer = gtk_menu_item_new_with_mnemonic(_("Thumbnail Viewer"));
+    window->priv->menus.view.menu_item_show_toolbar = gtk_check_menu_item_new_with_mnemonic(_("Show _Toolbar"));
+    window->priv->menus.view.menu_item_show_thumbnail_viewer = gtk_menu_item_new_with_mnemonic(_("Thumbnail _Viewer"));
+
+    window->priv->menus.view.menu_item_separator_1 = gtk_separator_menu_item_new();
+
+    window->priv->menus.view.menu_item_zooming = gtk_menu_item_new_with_mnemonic(_("_Zooming"));
+    window->priv->menus.view.menu_item_rotate = gtk_menu_item_new_with_mnemonic(_("_Rotate"));
+
+    window->priv->menus.view.menu_item_separator_2 = gtk_separator_menu_item_new();
+
     window->priv->menus.view.menu_item_fullscreen = gtk_image_menu_item_new_from_stock(GTK_STOCK_FULLSCREEN, NULL);
 
     gtk_widget_add_accelerator(window->priv->menus.view.menu_item_fullscreen, "activate", accel_group, GDK_F11, 0,GTK_ACCEL_VISIBLE);
@@ -361,8 +393,13 @@ rstto_main_window_init(RsttoMainWindow *window)
 
     gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.view.menu), window->priv->menus.view.menu_item_show_toolbar);
     gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.view.menu), window->priv->menus.view.menu_item_show_thumbnail_viewer);
+    gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.view.menu), window->priv->menus.view.menu_item_separator_1);
+    gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.view.menu), window->priv->menus.view.menu_item_zooming);
+    gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.view.menu), window->priv->menus.view.menu_item_rotate);
+    gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.view.menu), window->priv->menus.view.menu_item_separator_2);
     gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.view.menu), window->priv->menus.view.menu_item_fullscreen);
 
+/* Create 'View/Show thumbnail-bar' menu */
     window->priv->menus.view.show_thumbnail_viewer.menu = gtk_menu_new();
     gtk_menu_set_accel_group(GTK_MENU(window->priv->menus.view.show_thumbnail_viewer.menu), accel_group);
 
@@ -371,15 +408,15 @@ rstto_main_window_init(RsttoMainWindow *window)
     window->priv->menus.view.show_thumbnail_viewer.
             menu_item_thumbnail_viewer_horizontal = gtk_radio_menu_item_new_with_mnemonic(
                     NULL, 
-                    _("Show Horizontally"));
+                    _("Show _Horizontally"));
     window->priv->menus.view.show_thumbnail_viewer.
             menu_item_thumbnail_viewer_vertical = gtk_radio_menu_item_new_with_mnemonic_from_widget(
                     GTK_RADIO_MENU_ITEM(window->priv->menus.view.show_thumbnail_viewer.menu_item_thumbnail_viewer_horizontal),
-                    _("Show Vertically"));
+                    _("Show _Vertically"));
     window->priv->menus.view.show_thumbnail_viewer.
             menu_item_thumbnail_viewer_hide = gtk_radio_menu_item_new_with_mnemonic_from_widget(
                     GTK_RADIO_MENU_ITEM(window->priv->menus.view.show_thumbnail_viewer.menu_item_thumbnail_viewer_horizontal),
-                    _("Hide"));
+                    _("H_ide"));
 
     gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.view.show_thumbnail_viewer.menu),
             window->priv->menus.view.show_thumbnail_viewer.menu_item_thumbnail_viewer_horizontal);
@@ -388,6 +425,42 @@ rstto_main_window_init(RsttoMainWindow *window)
     gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.view.show_thumbnail_viewer.menu),
             window->priv->menus.view.show_thumbnail_viewer.menu_item_thumbnail_viewer_hide);
 
+/* Create 'view/zooming' menu */
+    window->priv->menus.view.zooming.menu = gtk_menu_new();
+    gtk_menu_set_accel_group(GTK_MENU(window->priv->menus.view.zooming.menu), accel_group);
+
+    gtk_menu_item_set_submenu(GTK_MENU_ITEM(window->priv->menus.view.menu_item_zooming),
+            window->priv->menus.view.zooming.menu);
+
+    window->priv->menus.view.zooming.menu_item_zoom_in = gtk_image_menu_item_new_from_stock(GTK_STOCK_ZOOM_IN, accel_group);
+    window->priv->menus.view.zooming.menu_item_zoom_out = gtk_image_menu_item_new_from_stock(GTK_STOCK_ZOOM_OUT, accel_group);
+    window->priv->menus.view.zooming.menu_item_zoom_100 = gtk_image_menu_item_new_from_stock(GTK_STOCK_ZOOM_100, accel_group);
+    window->priv->menus.view.zooming.menu_item_zoom_fit = gtk_image_menu_item_new_from_stock(GTK_STOCK_ZOOM_FIT, accel_group);
+
+    gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.view.zooming.menu),
+            window->priv->menus.view.zooming.menu_item_zoom_in);
+    gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.view.zooming.menu),
+            window->priv->menus.view.zooming.menu_item_zoom_out);
+    gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.view.zooming.menu),
+            window->priv->menus.view.zooming.menu_item_zoom_100);
+    gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.view.zooming.menu),
+            window->priv->menus.view.zooming.menu_item_zoom_fit);
+
+/* Create 'view/rotate' menu */
+    window->priv->menus.view.rotate.menu = gtk_menu_new();
+    gtk_menu_set_accel_group(GTK_MENU(window->priv->menus.view.rotate.menu), accel_group);
+
+    gtk_menu_item_set_submenu(GTK_MENU_ITEM(window->priv->menus.view.menu_item_rotate),
+            window->priv->menus.view.rotate.menu);
+
+    window->priv->menus.view.rotate.menu_item_rotate_cw = gtk_image_menu_item_new_with_mnemonic(_("Rotate _Right"));
+    window->priv->menus.view.rotate.menu_item_rotate_ccw = gtk_image_menu_item_new_with_mnemonic(_("Rotate _Left"));
+
+    gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.view.rotate.menu),
+            window->priv->menus.view.rotate.menu_item_rotate_cw);
+    gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.view.rotate.menu),
+            window->priv->menus.view.rotate.menu_item_rotate_ccw);
+
 /* Create 'Go' menu */
     window->priv->menus.menu_item_go= gtk_menu_item_new_with_mnemonic(_("_Go"));
     gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.menu), window->priv->menus.menu_item_go);
@@ -462,8 +535,16 @@ rstto_main_window_init(RsttoMainWindow *window)
     gtk_widget_set_sensitive(GTK_WIDGET(window->priv->toolbar.tool_item_next), FALSE);
     gtk_widget_set_sensitive(GTK_WIDGET(window->priv->toolbar.tool_item_zoom_in), FALSE);
     gtk_widget_set_sensitive(GTK_WIDGET(window->priv->toolbar.tool_item_zoom_out), FALSE);
-    gtk_widget_set_sensitive(GTK_WIDGET(window->priv->toolbar.tool_item_zoom_fit), FALSE);
     gtk_widget_set_sensitive(GTK_WIDGET(window->priv->toolbar.tool_item_zoom_100), FALSE);
+    gtk_widget_set_sensitive(GTK_WIDGET(window->priv->toolbar.tool_item_zoom_fit), FALSE);
+
+    gtk_widget_set_sensitive(GTK_WIDGET(window->priv->menus.view.zooming.menu_item_zoom_in), FALSE);
+    gtk_widget_set_sensitive(GTK_WIDGET(window->priv->menus.view.zooming.menu_item_zoom_out), FALSE);
+    gtk_widget_set_sensitive(GTK_WIDGET(window->priv->menus.view.zooming.menu_item_zoom_100), FALSE);
+    gtk_widget_set_sensitive(GTK_WIDGET(window->priv->menus.view.zooming.menu_item_zoom_fit), FALSE);
+
+    gtk_widget_set_sensitive(GTK_WIDGET(window->priv->menus.view.rotate.menu_item_rotate_cw), FALSE);
+    gtk_widget_set_sensitive(GTK_WIDGET(window->priv->menus.view.rotate.menu_item_rotate_ccw), FALSE);
 
 /* Create statusbar */
     window->priv->statusbar = gtk_statusbar_new();
@@ -540,10 +621,13 @@ rstto_main_window_init(RsttoMainWindow *window)
 /* D-Bus stuff */
 
     window->priv->connection = dbus_g_bus_get(DBUS_BUS_SESSION, NULL);
-    window->priv->filemanager_proxy = dbus_g_proxy_new_for_name(window->priv->connection,
+    if (window->priv->connection)
+    {
+        window->priv->filemanager_proxy = dbus_g_proxy_new_for_name(window->priv->connection,
                                                                 "org.xfce.FileManager",
                                                                 "/org/xfce/FileManager",
                                                                 "org.xfce.FileManager");
+    }
 
 /* Connect signals */
     
@@ -614,6 +698,28 @@ rstto_main_window_init(RsttoMainWindow *window)
             "activate",
             G_CALLBACK(cb_rstto_main_window_preferences), window);
 
+/* zoom menu items */
+    g_signal_connect(window->priv->menus.view.zooming.menu_item_zoom_in,
+            "activate",
+            G_CALLBACK(cb_rstto_main_window_zoom_in), window);
+    g_signal_connect(window->priv->menus.view.zooming.menu_item_zoom_out,
+            "activate",
+            G_CALLBACK(cb_rstto_main_window_zoom_out), window);
+    g_signal_connect(window->priv->menus.view.zooming.menu_item_zoom_100,
+            "activate",
+            G_CALLBACK(cb_rstto_main_window_zoom_100), window);
+    g_signal_connect(window->priv->menus.view.zooming.menu_item_zoom_fit,
+            "activate",
+            G_CALLBACK(cb_rstto_main_window_zoom_fit), window);
+/* rotate menu items */
+    g_signal_connect(window->priv->menus.view.rotate.menu_item_rotate_cw,
+            "activate",
+            G_CALLBACK(cb_rstto_main_window_rotate_cw), window);
+    g_signal_connect(window->priv->menus.view.rotate.menu_item_rotate_ccw,
+            "activate",
+            G_CALLBACK(cb_rstto_main_window_rotate_ccw), window);
+
+/* go menu items */
     g_signal_connect(window->priv->menus.go.menu_item_next,
             "activate",
             G_CALLBACK(cb_rstto_main_window_next), window);
@@ -977,11 +1083,16 @@ cb_rstto_main_window_state_event(GtkWidget *widget, GdkEventWindowState *event,
             gtk_widget_hide(window->priv->menus.menu);
             gtk_widget_hide(window->priv->toolbar.bar);
             gtk_widget_hide(window->priv->statusbar);
+            GdkColor *color = g_new0(GdkColor, 1);
+            color->pixel = 0;
+
+            rstto_picture_viewer_set_bg_color(RSTTO_PICTURE_VIEWER(window->priv->picture_viewer), color);
         }
         else
         {
             gtk_widget_show(window->priv->menus.menu);
             gtk_widget_show(window->priv->statusbar);
+            rstto_picture_viewer_set_bg_color(RSTTO_PICTURE_VIEWER(window->priv->picture_viewer), NULL);
 
             if (window->priv->settings.toolbar_visibility == TRUE)
             {
@@ -1350,6 +1461,7 @@ cb_rstto_main_window_nav_iter_changed(RsttoNavigator *navigator, gint nr, RsttoN
         gtk_widget_set_sensitive(window->priv->menus.go.menu_item_next, TRUE);
         gtk_widget_set_sensitive(window->priv->menus.go.menu_item_play, TRUE);
         gtk_widget_set_sensitive(window->priv->menus.go.menu_item_pause, TRUE);
+
         gtk_widget_set_sensitive(GTK_WIDGET(window->priv->toolbar.tool_item_next), TRUE);
         gtk_widget_set_sensitive(GTK_WIDGET(window->priv->toolbar.tool_item_previous), TRUE);
 
@@ -1364,6 +1476,15 @@ cb_rstto_main_window_nav_iter_changed(RsttoNavigator *navigator, gint nr, RsttoN
         gtk_widget_set_sensitive(GTK_WIDGET(window->priv->menus._picture_viewer.menu_item_zoom_100), TRUE);
         gtk_widget_set_sensitive(GTK_WIDGET(window->priv->menus._picture_viewer.menu_item_zoom_fit), TRUE);
 
+        gtk_widget_set_sensitive(GTK_WIDGET(window->priv->menus.view.zooming.menu_item_zoom_in), TRUE);
+        gtk_widget_set_sensitive(GTK_WIDGET(window->priv->menus.view.zooming.menu_item_zoom_out), TRUE);
+        gtk_widget_set_sensitive(GTK_WIDGET(window->priv->menus.view.zooming.menu_item_zoom_100), TRUE);
+        gtk_widget_set_sensitive(GTK_WIDGET(window->priv->menus.view.zooming.menu_item_zoom_fit), TRUE);
+
+        gtk_widget_set_sensitive(GTK_WIDGET(window->priv->menus.view.rotate.menu_item_rotate_cw), TRUE);
+        gtk_widget_set_sensitive(GTK_WIDGET(window->priv->menus.view.rotate.menu_item_rotate_ccw), TRUE);
+
+
         /* Update window title */
         if (rstto_navigator_get_n_files(navigator) > 1)
         {
@@ -1462,6 +1583,14 @@ cb_rstto_main_window_nav_iter_changed(RsttoNavigator *navigator, gint nr, RsttoN
             gtk_widget_set_sensitive(GTK_WIDGET(window->priv->menus._picture_viewer.menu_item_zoom_out), FALSE);
             gtk_widget_set_sensitive(GTK_WIDGET(window->priv->menus._picture_viewer.menu_item_zoom_100), FALSE);
             gtk_widget_set_sensitive(GTK_WIDGET(window->priv->menus._picture_viewer.menu_item_zoom_fit), FALSE);
+
+            gtk_widget_set_sensitive(GTK_WIDGET(window->priv->menus.view.zooming.menu_item_zoom_in), FALSE);
+            gtk_widget_set_sensitive(GTK_WIDGET(window->priv->menus.view.zooming.menu_item_zoom_out), FALSE);
+            gtk_widget_set_sensitive(GTK_WIDGET(window->priv->menus.view.zooming.menu_item_zoom_100), FALSE);
+            gtk_widget_set_sensitive(GTK_WIDGET(window->priv->menus.view.zooming.menu_item_zoom_fit), FALSE);
+
+            gtk_widget_set_sensitive(GTK_WIDGET(window->priv->menus.view.rotate.menu_item_rotate_cw), FALSE);
+            gtk_widget_set_sensitive(GTK_WIDGET(window->priv->menus.view.rotate.menu_item_rotate_ccw), FALSE);
         }
 
         gtk_container_foreach(GTK_CONTAINER(window->priv->menus.edit.open_with.menu), (GtkCallback)gtk_widget_destroy, NULL);
@@ -1560,6 +1689,64 @@ cb_rstto_main_window_zoom_fit(GtkWidget *widget, RsttoMainWindow *window)
     rstto_picture_viewer_fit_scale(RSTTO_PICTURE_VIEWER(window->priv->picture_viewer));
 }
 
+static void
+cb_rstto_main_window_rotate_cw(GtkWidget *widget, RsttoMainWindow *window)
+{
+    RsttoNavigatorEntry *entry = rstto_navigator_get_file(window->priv->navigator);
+    if (entry)
+    {
+        GdkPixbufRotation rotation = rstto_navigator_entry_get_rotation(entry);
+        switch (rotation)
+        {
+            case GDK_PIXBUF_ROTATE_NONE:
+                rotation = GDK_PIXBUF_ROTATE_CLOCKWISE;
+                break;
+            case GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE:
+                rotation = GDK_PIXBUF_ROTATE_NONE;
+                break;
+            case GDK_PIXBUF_ROTATE_UPSIDEDOWN:
+                rotation = GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE;
+                break;
+            case GDK_PIXBUF_ROTATE_CLOCKWISE:
+                rotation = GDK_PIXBUF_ROTATE_UPSIDEDOWN;
+                break;
+            default:
+                g_warning("Unable to rotate, rotation unknown");
+                break;
+        }
+        rstto_navigator_entry_set_rotation(entry, rotation);
+    }
+}
+
+static void
+cb_rstto_main_window_rotate_ccw(GtkWidget *widget, RsttoMainWindow *window)
+{
+    RsttoNavigatorEntry *entry = rstto_navigator_get_file(window->priv->navigator);
+    if (entry)
+    {
+        GdkPixbufRotation rotation = rstto_navigator_entry_get_rotation(entry);
+        switch (rotation)
+        {
+            case GDK_PIXBUF_ROTATE_NONE:
+                rotation = GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE;
+                break;
+            case GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE:
+                rotation = GDK_PIXBUF_ROTATE_UPSIDEDOWN;
+                break;
+            case GDK_PIXBUF_ROTATE_UPSIDEDOWN:
+                rotation = GDK_PIXBUF_ROTATE_CLOCKWISE;
+                break;
+            case GDK_PIXBUF_ROTATE_CLOCKWISE:
+                rotation = GDK_PIXBUF_ROTATE_NONE;
+                break;
+            default:
+                g_warning("Unable to rotate, rotation unknown");
+                break;
+        }
+        rstto_navigator_entry_set_rotation(entry, rotation);
+    }
+}
+
 static void
 cb_rstto_main_window_spawn_app(GtkWidget *widget, ThunarVfsMimeApplication *app)
 {
