@@ -24,6 +24,13 @@
 #include "navigator.h"
 #include "picture_viewer.h"
 
+typedef enum
+{
+    RSTTO_PICTURE_VIEWER_STATE_NONE,
+    RSTTO_PICTURE_VIEWER_STATE_MOVE,
+    RSTTO_PICTURE_VIEWER_STATE_BOX_ZOOM
+} RsttoPictureViewerState;
+
 
 struct _RsttoPictureViewerPriv
 {
@@ -34,6 +41,7 @@ struct _RsttoPictureViewerPriv
     GdkPixbuf        *dst_pixbuf; /* The pixbuf which ends up on screen */
     void             (*cb_value_changed)(GtkAdjustment *, RsttoPictureViewer *);
     gboolean          show_border;
+    GdkColor         *bg_color;
     struct
     {
         gdouble x;
@@ -42,6 +50,7 @@ struct _RsttoPictureViewerPriv
         gdouble current_y;
         gint h_val;
         gint v_val;
+        RsttoPictureViewerState state;
     } motion;
     struct
     {
@@ -267,7 +276,14 @@ rstto_picture_viewer_paint(GtkWidget *widget)
         GdkPixmap *buffer = gdk_pixmap_new(NULL, widget->allocation.width, widget->allocation.height, gdk_drawable_get_depth(widget->window));
         GdkGC *gc = gdk_gc_new(GDK_DRAWABLE(buffer));
 
-        gdk_gc_set_foreground(gc, &color);
+        if (viewer->priv->bg_color)
+        {
+            gdk_gc_set_foreground(gc, viewer->priv->bg_color);
+        }
+        else
+        {
+            gdk_gc_set_foreground(gc, &(widget->style->bg[GTK_STATE_NORMAL]));
+        }
         gdk_draw_rectangle(GDK_DRAWABLE(buffer), gc, TRUE, 0, 0, widget->allocation.width, widget->allocation.height);
         if(pixbuf)
         {
@@ -326,6 +342,41 @@ rstto_picture_viewer_paint(GtkWidget *widget)
                             y2,
                             GDK_RGB_DITHER_NONE,
                             0,0);
+            if(viewer->priv->motion.state == RSTTO_PICTURE_VIEWER_STATE_BOX_ZOOM)
+            {
+                gdk_gc_set_foreground(gc,
+                        &(widget->style->fg[GTK_STATE_SELECTED]));
+                gdouble x1, x2, y1, y2;
+
+                if (viewer->priv->motion.x < viewer->priv->motion.current_x)
+                {
+                    x1 = viewer->priv->motion.x;
+                    x2 = viewer->priv->motion.current_x;
+                }
+                else
+                {
+                    x1 = viewer->priv->motion.current_x;
+                    x2 = viewer->priv->motion.x;
+                }
+                if (viewer->priv->motion.y < viewer->priv->motion.current_y)
+                {
+                    y1 = viewer->priv->motion.y;
+                    y2 = viewer->priv->motion.current_y;
+                }
+                else
+                {
+                    y1 = viewer->priv->motion.current_y;
+                    y2 = viewer->priv->motion.y;
+                }
+                gdk_draw_rectangle(GDK_DRAWABLE(buffer),
+                                gc,
+                                FALSE,
+                                x1,
+                                y1,
+                                x2 - x1,
+                                y2 - y1);
+            }
+
             if(viewer->priv->show_border)
             {
                 gdk_gc_set_foreground(gc, &line_color);
@@ -750,36 +801,46 @@ cb_rstto_picture_viewer_motion_notify_event (RsttoPictureViewer *viewer,
             viewer->priv->refresh.idle_id = 0;
         }
 
-        if (viewer->priv->motion.x != viewer->priv->motion.current_x)
+        switch (viewer->priv->motion.state)
         {
-            gint val = viewer->hadjustment->value;
-            viewer->hadjustment->value = viewer->priv->motion.h_val + (viewer->priv->motion.x - viewer->priv->motion.current_x);
-            if((viewer->hadjustment->value + viewer->hadjustment->page_size) > viewer->hadjustment->upper)
-            {
-                viewer->hadjustment->value = viewer->hadjustment->upper - viewer->hadjustment->page_size;
-            }
-            if((viewer->hadjustment->value) < viewer->hadjustment->lower)
-            {
-                viewer->hadjustment->value = viewer->hadjustment->lower;
-            }
-            if (val != viewer->hadjustment->value)
-                gtk_adjustment_value_changed(viewer->hadjustment);
-        }
+            case RSTTO_PICTURE_VIEWER_STATE_MOVE:
+                if (viewer->priv->motion.x != viewer->priv->motion.current_x)
+                {
+                    gint val = viewer->hadjustment->value;
+                    viewer->hadjustment->value = viewer->priv->motion.h_val + (viewer->priv->motion.x - viewer->priv->motion.current_x);
+                    if((viewer->hadjustment->value + viewer->hadjustment->page_size) > viewer->hadjustment->upper)
+                    {
+                        viewer->hadjustment->value = viewer->hadjustment->upper - viewer->hadjustment->page_size;
+                    }
+                    if((viewer->hadjustment->value) < viewer->hadjustment->lower)
+                    {
+                        viewer->hadjustment->value = viewer->hadjustment->lower;
+                    }
+                    if (val != viewer->hadjustment->value)
+                        gtk_adjustment_value_changed(viewer->hadjustment);
+                }
 
-        if (viewer->priv->motion.y != viewer->priv->motion.current_y)
-        {
-            gint val = viewer->vadjustment->value;
-            viewer->vadjustment->value = viewer->priv->motion.v_val + (viewer->priv->motion.y - viewer->priv->motion.current_y);
-            if((viewer->vadjustment->value + viewer->vadjustment->page_size) > viewer->vadjustment->upper)
-            {
-                viewer->vadjustment->value = viewer->vadjustment->upper - viewer->vadjustment->page_size;
-            }
-            if((viewer->vadjustment->value) < viewer->vadjustment->lower)
-            {
-                viewer->vadjustment->value = viewer->vadjustment->lower;
-            }
-            if (val != viewer->vadjustment->value)
-                gtk_adjustment_value_changed(viewer->vadjustment);
+                if (viewer->priv->motion.y != viewer->priv->motion.current_y)
+                {
+                    gint val = viewer->vadjustment->value;
+                    viewer->vadjustment->value = viewer->priv->motion.v_val + (viewer->priv->motion.y - viewer->priv->motion.current_y);
+                    if((viewer->vadjustment->value + viewer->vadjustment->page_size) > viewer->vadjustment->upper)
+                    {
+                        viewer->vadjustment->value = viewer->vadjustment->upper - viewer->vadjustment->page_size;
+                    }
+                    if((viewer->vadjustment->value) < viewer->vadjustment->lower)
+                    {
+                        viewer->vadjustment->value = viewer->vadjustment->lower;
+                    }
+                    if (val != viewer->vadjustment->value)
+                        gtk_adjustment_value_changed(viewer->vadjustment);
+                }
+                break;
+            case RSTTO_PICTURE_VIEWER_STATE_BOX_ZOOM:
+                viewer->priv->refresh.idle_id = g_idle_add((GSourceFunc)cb_rstto_picture_viewer_queued_repaint, viewer);
+                break;
+            default:
+                break;
         }
     }
     return FALSE;
@@ -800,15 +861,36 @@ cb_rstto_picture_viewer_button_press_event (RsttoPictureViewer *viewer, GdkEvent
 {
     if(event->button == 1)
     {
-        GtkWidget *widget = GTK_WIDGET(viewer);
-        GdkCursor *cursor = gdk_cursor_new(GDK_FLEUR);
-        gdk_window_set_cursor(widget->window, cursor);
-        gdk_cursor_unref(cursor);
-        
         viewer->priv->motion.x = event->x;
         viewer->priv->motion.y = event->y;
         viewer->priv->motion.h_val = viewer->hadjustment->value;
         viewer->priv->motion.v_val = viewer->vadjustment->value;
+
+        if (rstto_navigator_get_file(viewer->priv->navigator) != NULL)
+        {
+
+            if (!(event->state & GDK_MODIFIER_MASK))
+            {
+                GtkWidget *widget = GTK_WIDGET(viewer);
+                GdkCursor *cursor = gdk_cursor_new(GDK_FLEUR);
+                gdk_window_set_cursor(widget->window, cursor);
+                gdk_cursor_unref(cursor);
+
+                viewer->priv->motion.state = RSTTO_PICTURE_VIEWER_STATE_MOVE;
+            }
+
+            if (event->state & GDK_CONTROL_MASK)
+            {
+                GtkWidget *widget = GTK_WIDGET(viewer);
+                GdkCursor *cursor = gdk_cursor_new(GDK_UL_ANGLE);
+                gdk_window_set_cursor(widget->window, cursor);
+                gdk_cursor_unref(cursor);
+
+                viewer->priv->motion.state = RSTTO_PICTURE_VIEWER_STATE_BOX_ZOOM;
+            }
+        }
+
+        
     }
     if(event->button == 3)
     {
@@ -833,6 +915,109 @@ cb_rstto_picture_viewer_button_release_event (RsttoPictureViewer *viewer, GdkEve
     {
         GtkWidget *widget = GTK_WIDGET(viewer);
         gdk_window_set_cursor(widget->window, NULL);
+        switch (viewer->priv->motion.state)
+        {
+            case RSTTO_PICTURE_VIEWER_STATE_NONE:
+                break;
+            case RSTTO_PICTURE_VIEWER_STATE_MOVE:
+                break;
+            case RSTTO_PICTURE_VIEWER_STATE_BOX_ZOOM:
+                if(GTK_WIDGET_REALIZED(widget))
+                {
+                    RsttoNavigatorEntry *entry = rstto_navigator_get_file(viewer->priv->navigator);
+                    gdouble scale = rstto_navigator_entry_get_scale(entry);
+                    gdouble old_scale = scale;
+                    gdouble width = (gdouble)gdk_pixbuf_get_width(viewer->priv->src_pixbuf);
+                    gdouble height = (gdouble)gdk_pixbuf_get_height(viewer->priv->src_pixbuf);
+                    gdouble box_width, box_height;
+                    gdouble top_left_x, top_left_y;
+                    if (viewer->priv->motion.x < viewer->priv->motion.current_x)
+                    {
+                        top_left_x = viewer->priv->motion.x + viewer->hadjustment->value;
+                        box_width = viewer->priv->motion.current_x - viewer->priv->motion.x;
+                    }
+                    else
+                    {
+                        top_left_x = viewer->priv->motion.current_x + viewer->hadjustment->value;
+                        box_width = viewer->priv->motion.x - viewer->priv->motion.current_x;
+                    }
+                    if (viewer->priv->motion.y < viewer->priv->motion.current_y)
+                    {
+                        top_left_y = viewer->priv->motion.y + viewer->vadjustment->value;
+                        box_height = viewer->priv->motion.current_y - viewer->priv->motion.y;
+                    }
+                    else
+                    {
+                        top_left_y = viewer->priv->motion.current_y + viewer->vadjustment->value;
+                        box_height = viewer->priv->motion.y - viewer->priv->motion.current_y;
+                    }
+
+                    gdouble h_scale = widget->allocation.width / box_width * scale;
+                    gdouble v_scale = widget->allocation.height / box_height * scale;
+
+                    if (h_scale < v_scale)
+                    {
+                        rstto_navigator_entry_set_scale(entry, h_scale);
+                    }
+                    else
+                    {
+                        rstto_navigator_entry_set_scale(entry, v_scale);
+                    }
+
+                    rstto_navigator_entry_set_fit_to_screen(entry, FALSE);
+                    scale = rstto_navigator_entry_get_scale(entry);
+
+
+                        
+                    if(viewer->hadjustment)
+                    {
+                        viewer->hadjustment->page_size = box_width / old_scale * scale;
+                        viewer->hadjustment->upper = width * scale;
+                        viewer->hadjustment->lower = 0;
+                        viewer->hadjustment->step_increment = 1;
+                        viewer->hadjustment->page_increment = 100;
+                        viewer->hadjustment->value = top_left_x;
+                        if((viewer->hadjustment->value + viewer->hadjustment->page_size) > viewer->hadjustment->upper)
+                        {
+                            viewer->hadjustment->value = viewer->hadjustment->upper - viewer->hadjustment->page_size;
+                        }
+                        if(viewer->hadjustment->value < viewer->hadjustment->lower)
+                        {
+                            viewer->hadjustment->value = viewer->hadjustment->lower;
+                        }
+                        gtk_adjustment_changed(viewer->hadjustment);
+                        gtk_adjustment_value_changed(viewer->hadjustment);
+                    }
+                    if(viewer->vadjustment)
+                    {
+                        viewer->vadjustment->page_size = box_height /old_scale* scale;
+                        viewer->vadjustment->upper = height * scale;
+                        viewer->vadjustment->lower = 0;
+                        viewer->vadjustment->step_increment = 1;
+                        viewer->vadjustment->page_increment = 100;
+                        viewer->vadjustment->value = top_left_y;
+                        if((viewer->vadjustment->value + viewer->vadjustment->page_size) > viewer->vadjustment->upper)
+                        {
+                            viewer->vadjustment->value = viewer->vadjustment->upper - viewer->vadjustment->page_size;
+                        }
+                        if(viewer->vadjustment->value < viewer->vadjustment->lower)
+                        {
+                            viewer->vadjustment->value = viewer->vadjustment->lower;
+                        }
+                        gtk_adjustment_changed(viewer->vadjustment);
+                        gtk_adjustment_value_changed(viewer->vadjustment);
+                    }
+                }
+                viewer->priv->motion.state = RSTTO_PICTURE_VIEWER_STATE_NONE;
+
+                if (viewer->priv->refresh.idle_id > 0)
+                {
+                    g_source_remove(viewer->priv->refresh.idle_id);
+                }
+                viewer->priv->refresh.idle_id = g_idle_add((GSourceFunc)cb_rstto_picture_viewer_queued_repaint, viewer);
+                break;
+        }
+
     }
 
 }
@@ -869,3 +1054,17 @@ rstto_picture_viewer_set_menu (RsttoPictureViewer *viewer, GtkMenu *menu)
         gtk_menu_attach_to_widget(viewer->priv->menu, GTK_WIDGET(viewer), NULL);
     }
 }
+
+void
+rstto_picture_viewer_set_bg_color (RsttoPictureViewer *viewer, GdkColor *color)
+{
+    if (viewer->priv->bg_color)
+    {
+        gdk_color_free(viewer->priv->bg_color);
+        viewer->priv->bg_color = NULL;
+    }
+    if (color)
+    {
+        viewer->priv->bg_color = gdk_color_copy(color);
+    }
+}
